// $URL: http://subversion:8080/svn/gsc/trunk/drivers/gsc_common/driver/gsc_io.c $
// $Rev: 21125 $
// $Date: 2013-04-19 13:01:39 -0500 (Fri, 19 Apr 2013) $

#include "main.h"



/******************************************************************************
*
*	Function:	gsc_io_create
*
*	Purpose:
*
*		Perform I/O based initialization as the driver is being loaded.
*
*	Arguments:
*
*		alt		The structure for the device to access.
*
*		io		The I/O structure of interest.
*
*		size	The desired size of the buffer in bytes.
*
*	Returned:
*
*		0		All went well.
*		< 0		An appropriate error code.
*
******************************************************************************/

int gsc_io_create(GSC_ALT_STRUCT_T* alt, dev_io_t* io, size_t bytes)
{
	int	ret;

	os_sem_create(&io->sem);
	io->bytes	= bytes;
	io->buf		= os_mem_dma_alloc(&io->bytes, &io->mem);
	ret			= io->buf ? 0 : -ENOMEM;
	return(ret);
}



/******************************************************************************
*
*	Function:	gsc_io_destroy
*
*	Purpose:
*
*		Perform I/O based cleanup as the driver is being unloaded.
*
*	Arguments:
*
*		alt		The structure for the device to access.
*
*		io		The I/O structure of interest.
*
*	Returned:
*
*		None.
*
******************************************************************************/

void gsc_io_destroy(GSC_ALT_STRUCT_T* alt, dev_io_t* io)
{
	if (io->buf)
		os_mem_dma_free(&io->mem);

	io->buf		= NULL;
	io->bytes	= 0;
	os_sem_destroy(&io->sem);
	memset(io, 0, sizeof(dev_io_t));
}


