// $URL: http://subversion:8080/svn/gsc/trunk/drivers/gsc_common/driver/intime/os_main.h $
// $Rev: 26098 $
// $Date: 2014-05-16 10:50:37 -0500 (Fri, 16 May 2014) $

// INtime 4 driver

#ifndef __OS_MAIN_H__
#define __OS_MAIN_H__

#include <errno.h>
#include <stdio.h>
#include <string.h>

#include "rt.h"
#include "pcibus.h"



// #defines *******************************************************************

// This is for the OS specific code.
#define	OS_COMMON_VERSION			"3"
// 3	Corrected an interrupt acquisition bug.
// 2	LINTed sources.
// 1	Made a correction to the code for exiting the driver.
//		Added definition of ERESTARTSYS.
//		Updated the code for acquiring and releasing the interrupt.
//		Updated the PCI Enable/Disable code.
//		Improved some error reporting.
//		Updated semaphore processing code.
//		Removed the -u command line option.
// 0	Initial release.

#define	EXIT_UNCAT					0x01	// Uncatalog device mailboxes.

// Printing is unsupported in ISR.
#undef	PRINTF_ISR

#if ((!defined(GSC_DEVS_PER_BOARD)) || (GSC_DEVS_PER_BOARD < 1))
	#define	GSC_DEVS_PER_BOARD		1
#endif




// data types *****************************************************************

typedef QWORD	os_time_tick_t;
typedef	long	VADDR_T;

typedef struct					// Don't forget: _pci_region in gsc_reg.c
{
	int				index;		// BARx
	int				offset;		// Offset of BARx register in PCI space.
	u32				reg;		// Actual BARx register value.
	u32				flags;		// lower register bits
	int				io_mapped;	// Is this an I/O mapped region?
	unsigned long	phys_adrs;	// Physical address of region.
	u32				size;		// Region size in bytes.

	// These are computed when mapped in.
	u32				requested;	// Is resource requested from OS?
	VADDR_T			vaddr;		// Kernel virtual address.
} os_bar_t;

typedef struct
{
	void*			key;
	KNHANDLE		handle;
	u8				sem_data[KN_SEMAPHORE_SIZE];
} os_event_t;

typedef struct
{
	u8				active;
	WORD			irq_level;
} os_irq_t;

typedef struct
{
	void*			key;
	size_t			bytes;
	void*			alloc_ptr;	// DO NOT USE THIS FOR DMA!
	void*			ptr;
} os_mem_t;

typedef struct
{
	PCIDEV			pcidev;
} os_pci_t;

typedef	struct
{
	void*			key;
	KNHANDLE		handle;
	u8				sem_data[KN_SEMAPHORE_SIZE];
} os_sem_t;

typedef struct
{
	RTHANDLE			mbox;
	GSC_ALT_STRUCT_T*	alt;		// Used for shutdown.
	u8					initialized;
	u8					busy;
} os_service_data_t;	// Used by each thread that processes main messages.

typedef struct
{
	os_sem_t			sem;
	os_service_data_t	data[100];
} os_service_threads_t;	// Tracks service threads.

typedef struct
{
	int					in_use;
	GSC_ALT_STRUCT_T*	alt;
	int					index;
	RTHANDLE			mbox;
} os_dev_thread_data_t;	// Passes essential data to main device threads.

typedef struct
{
	os_sem_t				sem;
	os_dev_thread_data_t	data[100];
} os_dev_threads_t;

typedef struct
{
	QWORD			count;	// low level ticks
} os_time_t;

typedef struct
{
	char					built[32];
	int						dev_qty;
	int						driver_loaded;
	long					tick_rate;
	long					tick_us;

	os_dev_threads_t		dev_threads;
	os_service_threads_t	service_threads;

	// The list must be locked for opening and closing.
	os_sem_t				sem;
	dev_data_t*				dev_list[10];
} gsc_global_t;



// prototypes *****************************************************************

unsigned long	os_bar_pci_address(os_pci_t* pci, u8 index);
u32				os_bar_pci_flags(os_pci_t* pci, u8 index);
u32				os_bar_pci_size(os_pci_t* pci, u8 index);
int				os_bar_region_acquire(os_bar_t* bar);
void			os_bar_region_release(os_bar_t* bar);

int				os_close(GSC_ALT_STRUCT_T* alt);

int				os_driver_start(void);

void			os_event_cleanup(os_event_t* evnt);
void			os_event_init(os_event_t* evnt);
void			os_event_resume(os_event_t* evnt);
void			os_event_wait(os_event_t* evnt, os_time_tick_t timeout);
void			os_exit_driver(void);
void			os_exit_initiate(int flags);

int				os_init_driver(void);
int				os_ioctl(GSC_ALT_STRUCT_T* alt, long cmd, void* arg);
int				os_ioctl_init(void);
void			os_ioctl_reset(void);
int				os_irq_acquire(dev_data_t* dev);
void			os_irq_disable(dev_data_t* dev);
void			os_irq_enable(dev_data_t* dev);
void			os_irq_release(dev_data_t* dev);

int				os_mem_copy_from_user(void* dst, const void* src, long size);
int				os_mem_copy_to_user(void* dst, const void* src, long size);
void*			os_mem_data_alloc(size_t size);
void			os_mem_data_free(void* ptr);
void*			os_mem_dma_alloc(size_t* size, os_mem_t* mem);
void			os_mem_dma_free(os_mem_t* mem);
void*			os_mem_virt_to_phys(void* ptr);

int				os_open(GSC_ALT_STRUCT_T* alt);
int				os_pci_dev_disable(os_pci_t* pci);
int				os_pci_dev_enable(os_pci_t* pci);
void			os_pci_clear_master(os_pci_t* pci);
void			os_pci_set_master(os_pci_t* pci);

long			os_read(GSC_ALT_STRUCT_T* alt, void* buf, size_t count);
u8				os_reg_read_io_u8(dev_data_t* dev, VADDR_T va);
u8				os_reg_read_io_u8_imm(VADDR_T va);
u16				os_reg_read_io_u16(dev_data_t* dev, VADDR_T va);
u16				os_reg_read_io_u16_imm(VADDR_T va);
u32				os_reg_read_io_u32(dev_data_t* dev, VADDR_T va);
u32				os_reg_read_io_u32_imm(VADDR_T va);
u8				os_reg_read_mem_u8(dev_data_t* dev, VADDR_T va);
u8				os_reg_read_mem_u8_imm(VADDR_T va);
u16				os_reg_read_mem_u16(dev_data_t* dev, VADDR_T va);
u16				os_reg_read_mem_u16_imm(VADDR_T va);
u32				os_reg_read_mem_u32(dev_data_t* dev, VADDR_T va);
u32				os_reg_read_mem_u32_imm(VADDR_T va);
u8				os_reg_read_pci_u8(os_pci_t* pci, u16 offset);
u16				os_reg_read_pci_u16(os_pci_t* pci, u16 offset);
u32				os_reg_read_pci_u32(os_pci_t* pci, u16 offset);
void			os_reg_write_io_u8(dev_data_t* dev, VADDR_T va, u8 val);
void			os_reg_write_io_u8_imm(VADDR_T va, u8 val);
void			os_reg_write_io_u16(dev_data_t* dev, VADDR_T va, u16 val);
void			os_reg_write_io_u16_imm(VADDR_T va, u16 val);
void			os_reg_write_io_u32(dev_data_t* dev, VADDR_T va, u32 val);
void			os_reg_write_io_u32_imm(VADDR_T va, u32 val);
void			os_reg_write_mem_u8(dev_data_t* dev, VADDR_T va, u8 val);
void			os_reg_write_mem_u8_imm(VADDR_T va, u8 val);
void			os_reg_write_mem_u16(dev_data_t* dev, VADDR_T va, u16 val);
void			os_reg_write_mem_u16_imm(VADDR_T va, u16 val);
void			os_reg_write_mem_u32(dev_data_t* dev, VADDR_T va, u32 val);
void			os_reg_write_mem_u32_imm(VADDR_T va, u32 val);
void			os_reg_write_pci_u8(os_pci_t* pci, u16 offset, u8 val);
void			os_reg_write_pci_u16(os_pci_t* pci, u16 offset, u16 val);
void			os_reg_write_pci_u32(os_pci_t* pci, u16 offset, u32 val);
RTHANDLE		os_root_get_root_process(int* ret);
int				os_root_mbox_create(void);
void			os_root_mbox_service(void);
void			os_root_mbox_send_exit_msg(int flags);

int				os_sem_create(os_sem_t* sem);
int				os_sem_destroy(os_sem_t* sem);
int				os_sem_lock(os_sem_t* sem);
int				os_sem_unlock(os_sem_t* sem);

long			os_time_delta_ms(const os_time_t* tt1, const os_time_t* tt2);
void			os_time_get(os_time_t* tt);
os_time_tick_t	os_time_ms_to_ticks(long ms);
os_time_tick_t	os_time_tick_get(void);
int				os_time_tick_rate(void);
void			os_time_tick_sleep(int ticks);
int				os_time_tick_timedout(os_time_tick_t tick_time);
long			os_time_ticks_to_ms(os_time_tick_t ticks);
void			os_time_us_delay(long us);

long			os_write(GSC_ALT_STRUCT_T* alt, const void* buf, size_t count);



#endif
