// $URL: http://subversion:8080/svn/gsc/trunk/drivers/gsc_common/driver/intime/os_mem.c $
// $Rev: 23636 $
// $Date: 2013-10-22 11:25:51 -0500 (Tue, 22 Oct 2013) $

// INtime 4 driver

#include "main.h"



//*****************************************************************************
int os_mem_copy_from_user(void* dst, const void* src, long size)
{
	int	ret;

	if (dst && src)
	{
		// The memory pointers are both valid in the driver's address space.
		memcpy(dst, src, size);
		ret	= 0;
	}
	else
	{
		ret	= -EFAULT;
	}

	return(ret);
}



//*****************************************************************************
int os_mem_copy_to_user(void* dst, const void* src, long size)
{
	int	ret;

	if (dst && src)
	{
		// The memory pointers are both valid in the driver's address space.
		memcpy(dst, src, size);
		ret	= 0;
	}
	else
	{
		ret	= -EFAULT;
	}

	return(ret);
}



//*****************************************************************************
void* os_mem_data_alloc(size_t size)
{
	void*	ptr;

	if (size)
		ptr	= AllocateRtMemory(size);
	else
		ptr	= NULL;

	return(ptr);
}



//*****************************************************************************
void os_mem_data_free(void* ptr)
{
	if (ptr)
		FreeRtMemory(ptr);
}




//*****************************************************************************
void* os_mem_dma_alloc(size_t* bytes, os_mem_t* mem)
{
	DWORD	adrs;
	void*	dma;
	void*	ptr;

	for (;;)	// A convenience loop.
	{
		if ((bytes == NULL) || (mem == NULL))
		{
			// A pointer is NULL.
			ptr	= NULL;
			break;
		}

		if (mem->key == mem)
		{
			// The memory has already been allocated.
			ptr			= mem->ptr;
			bytes[0]	= mem->bytes;
			break;
		}

		if (bytes[0] == 0)
		{
			// No memory is being requested.
			memset(mem, 0, sizeof(mem[0]));
			ptr	= NULL;
			break;
		}

		ptr	= AllocateRtMemory(bytes[0]);

		if (ptr == NULL)
		{
			// The allocation failed.
			memset(mem, 0, sizeof(mem[0]));
			break;
		}

		adrs	= GetRtPhysicalAddress(ptr, bytes[0], 0);

		if (adrs == BAD_PHYSICAL_ADDRESS)
		{
			// We couldn't get the physical address.
			FreeRtMemory(ptr);
			ptr	= NULL;
			memset(mem, 0, sizeof(mem[0]));
			break;
		}

		dma	= MapRtPhysicalMemoryEx(adrs, bytes[0], PTE_CACHE_DISABLE);

		if (dma == NULL)
		{
			// We couldn't map the area as desired.
			FreeRtMemory(ptr);
			ptr	= NULL;
			memset(mem, 0, sizeof(mem[0]));
			break;
		}

		// The allocation and mapping was successful.
		mem->alloc_ptr	= ptr;
		mem->ptr		= dma;
		mem->bytes		= bytes[0];
		mem->key		= (void*) mem;
		break;
	}

	return(ptr);
}



//*****************************************************************************
void os_mem_dma_free(os_mem_t* mem)
{
	if (mem == NULL)
	{
		// The pointer is NULL.
	}
	else if (mem->key != mem)
	{
		// The memory has not been allocated.
	}
	else
	{
		FreeRtMemory(mem->ptr);			// Release the cache-free mapping.
		FreeRtMemory(mem->alloc_ptr);	// Release the memory block.
		memset(mem, 0, sizeof(mem[0]));
	}
}



//*****************************************************************************
void* os_mem_virt_to_phys(void* ptr)
{
	DWORD	dw;

	if (ptr == NULL)
	{
		dw	= 0;
	}
	else
	{
		dw	= GetRtPhysicalAddress(ptr, 1, 0);

		if (dw == BAD_PHYSICAL_ADDRESS)
			dw	= 0;
	}

	ptr	= (void*) dw;
	return(ptr);
}


