// $URL: http://subversion:8080/svn/gsc/trunk/drivers/gsc_common/driver/intime/os_reg.c $
// $Rev: 19616 $
// $Date: 2012-11-03 16:41:25 -0500 (Sat, 03 Nov 2012) $

// INtime 4 driver

#include "main.h"



//*****************************************************************************
u8 os_reg_read_io_u8(dev_data_t* dev, VADDR_T va)
{
	int	status;
	u8	val		= 0xFF;

	if (dev && va)
	{
		status	= gsc_irq_access_lock(dev, 0);
		val		= inbyte((WORD) va);

		if (status == 0)
			gsc_irq_access_unlock(dev, 0);
	}

	return(val);
}



//*****************************************************************************
u8 os_reg_read_io_u8_imm(VADDR_T va)
{
	u8	val	= 0xFF;

	if (va)
		val	= inbyte((WORD) va);

	return(val);
}



//*****************************************************************************
u16 os_reg_read_io_u16(dev_data_t* dev, VADDR_T va)
{
	int	status;
	u16	val		= 0xFFFF;

	if (dev && va)
	{
		status	= gsc_irq_access_lock(dev, 0);
		val		= inhword((WORD) va);

		if (status == 0)
			gsc_irq_access_unlock(dev, 0);
	}

	return(val);
}



//*****************************************************************************
u16 os_reg_read_io_u16_imm(VADDR_T va)
{
	u16	val	= 0xFFFF;

	if (va)
		val	= inhword((WORD) va);

	return(val);
}



//*****************************************************************************
u32 os_reg_read_io_u32(dev_data_t* dev, VADDR_T va)
{
	int	status;
	u32	val		= 0xFFFFFFFF;

	if (dev && va)
	{
		status	= gsc_irq_access_lock(dev, 0);
		val		= inword((WORD) va);

		if (status == 0)
			gsc_irq_access_unlock(dev, 0);
	}

	return(val);
}



//*****************************************************************************
u32 os_reg_read_io_u32_imm(VADDR_T va)
{
	u32	val	= 0xFFFFFFFF;

	if (va)
		val	= inword((WORD) va);

	return(val);
}



//*****************************************************************************
u8 os_reg_read_mem_u8(dev_data_t* dev, VADDR_T va)
{
	int	status;
	u8	val		= 0xFF;

	if (dev && va)
	{
		status	= gsc_irq_access_lock(dev, 0);
		val		= ((volatile u8*) va)[0];

		if (status == 0)
			gsc_irq_access_unlock(dev, 0);
	}

	return(val);
}



//*****************************************************************************
u8 os_reg_read_mem_u8_imm(VADDR_T va)
{
	u8	val	= 0xFF;

	if (va)
		val	= ((volatile u8*) va)[0];

	return(val);
}



//*****************************************************************************
u16 os_reg_read_mem_u16(dev_data_t* dev, VADDR_T va)
{
	int	status;
	u16	val		= 0xFFFF;

	if (dev && va)
	{
		status	= gsc_irq_access_lock(dev, 0);
		val		= ((volatile u16*) va)[0];

		if (status == 0)
			gsc_irq_access_unlock(dev, 0);
	}

	return(val);
}



//*****************************************************************************
u16 os_reg_read_mem_u16_imm(VADDR_T va)
{
	u16	val	= 0xFFFF;

	if (va)
		val	= ((volatile u16*) va)[0];

	return(val);
}



//*****************************************************************************
u32 os_reg_read_mem_u32(dev_data_t* dev, VADDR_T va)
{
	int	status;
	u32	val		= 0xFFFFFFFF;

	if (dev && va)
	{
		status	= gsc_irq_access_lock(dev, 0);
		val		= ((volatile u32*) va)[0];

		if (status == 0)
			gsc_irq_access_unlock(dev, 0);
	}

	return(val);
}



//*****************************************************************************
u32 os_reg_read_mem_u32_imm(VADDR_T va)
{
	u32	val	= 0xFFFFFFFF;

	if (va)
		val	= ((volatile u32*) va)[0];

	return(val);
}



//*****************************************************************************
u8 os_reg_read_pci_u8(os_pci_t* pci, u16 offset)
{
	DWORD	dw;
	u8		val	= 0xFF;

	if (pci)
	{
		dw	= PciGetConfigRegister(&pci->pcidev, (BYTE) offset, T_BYTE);
		val	= (u8) (dw & 0xFF);
	}

	return(val);
}



//*****************************************************************************
u16 os_reg_read_pci_u16(os_pci_t* pci, u16 offset)
{
	DWORD	dw;
	u16		val	= 0xFFFF;

	if (pci)
	{
		dw	= PciGetConfigRegister(&pci->pcidev, (BYTE) offset, T_WORD);
		val	= (u16) (dw & 0xFFFF);
	}

	return(val);
}



//*****************************************************************************
u32 os_reg_read_pci_u32(os_pci_t* pci, u16 offset)
{
	u32	val	= 0xFFFFFFFF;

	if (pci)
		val	= PciGetConfigRegister(&pci->pcidev, (BYTE) offset, T_DWORD);

	return(val);
}



//*****************************************************************************
void os_reg_write_io_u8(dev_data_t* dev, VADDR_T va, u8 val)
{
	int	status;

	if (dev && va)
	{
		status	= gsc_irq_access_lock(dev, 0);
		outbyte((WORD) va, val);

		if (status == 0)
			gsc_irq_access_unlock(dev, 0);
	}
}



//*****************************************************************************
void os_reg_write_io_u8_imm(VADDR_T va, u8 val)
{
	if (va)
		outbyte((WORD) va, val);
}



//*****************************************************************************
void os_reg_write_io_u16(dev_data_t* dev, VADDR_T va, u16 val)
{
	int	status;

	if (dev && va)
	{
		status	= gsc_irq_access_lock(dev, 0);
		outhword((WORD) va, val);

		if (status == 0)
			gsc_irq_access_unlock(dev, 0);
	}
}



//*****************************************************************************
void os_reg_write_io_u16_imm(VADDR_T va, u16 val)
{
	if (va)
		outhword((WORD) va, val);
}



//*****************************************************************************
void os_reg_write_io_u32(dev_data_t* dev, VADDR_T va, u32 val)
{
	int	status;

	if (dev && va)
	{
		status	= gsc_irq_access_lock(dev, 0);
		outword((WORD) va, val);

		if (status == 0)
			gsc_irq_access_unlock(dev, 0);
	}
}



//*****************************************************************************
void os_reg_write_io_u32_imm(VADDR_T va, u32 val)
{
	if (va)
		outword((WORD) va, val);
}



//*****************************************************************************
void os_reg_write_mem_u8(dev_data_t* dev, VADDR_T va, u8 val)
{
	int	status;

	if (dev && va)
	{
		status	= gsc_irq_access_lock(dev, 0);
		((volatile u8*) va)[0]	= val;

		if (status == 0)
			gsc_irq_access_unlock(dev, 0);
	}
}



//*****************************************************************************
void os_reg_write_mem_u8_imm(VADDR_T va, u8 val)
{
	if (va)
		((volatile u8*) va)[0]	= val;
}



//*****************************************************************************
void os_reg_write_mem_u16(dev_data_t* dev, VADDR_T va, u16 val)
{
	int	status;

	if (dev && va)
	{
		status	= gsc_irq_access_lock(dev, 0);
		((volatile u16*) va)[0]	= val;

		if (status == 0)
			gsc_irq_access_unlock(dev, 0);
	}
}



//*****************************************************************************
void os_reg_write_mem_u16_imm(VADDR_T va, u16 val)
{
	if (va)
		((volatile u16*) va)[0]	= val;
}



//*****************************************************************************
void os_reg_write_mem_u32(dev_data_t* dev, VADDR_T va, u32 val)
{
	int	status;

	if (dev && va)
	{
		status	= gsc_irq_access_lock(dev, 0);
		((volatile u32*) va)[0]	= val;

		if (status == 0)
			gsc_irq_access_unlock(dev, 0);
	}
}



//*****************************************************************************
void os_reg_write_mem_u32_imm(VADDR_T va, u32 val)
{
	if (va)
		((volatile u32*) va)[0]	= val;
}



//*****************************************************************************
void os_reg_write_pci_u8(os_pci_t* pci, u16 offset, u8 val)
{
	if (pci)
		PciSetConfigRegister(&pci->pcidev, (BYTE) offset, T_BYTE, val);
}



//*****************************************************************************
void os_reg_write_pci_u16(os_pci_t* pci, u16 offset, u16 val)
{
	if (pci)
		PciSetConfigRegister(&pci->pcidev, (BYTE) offset, T_WORD, val);
}



//*****************************************************************************
void os_reg_write_pci_u32(os_pci_t* pci, u16 offset, u32 val)
{
	if (pci)
		PciSetConfigRegister(&pci->pcidev, (BYTE) offset, T_DWORD, val);
}


