// $URL: http://subversion:8080/svn/gsc/trunk/drivers/gsc_common/driver/intime/os_sem.c $
// $Rev: 22965 $
// $Date: 2013-08-26 15:03:51 -0500 (Mon, 26 Aug 2013) $

// INtime 4 driver

#include "main.h"



//*****************************************************************************
int os_sem_create(os_sem_t* sem)
{
	int	ret;

	if (sem == NULL)
	{
		// The pointer is NULL.
		ret	= -EINVAL;
	}
	else if (sem->key == sem)
	{
		// The semaphore has already been created.
		ret			= 0;
	}
	else
	{
		// Create a new, one unit semaphore.
		sem->handle	= knCreateRtSemaphore((KNPTR) &sem->sem_data, KN_FIFO_QUEUING | KN_ONE_UNIT);
		sem->key	= (void*) sem;
		ret			= 0;
	}

	return(ret);
}



//*****************************************************************************
int os_sem_destroy(os_sem_t* sem)
{
	int	ret;

	if (sem == NULL)
	{
		// The pointer is NULL.
		ret	= -EINVAL;
	}
	else if (sem->key != sem)
	{
		// The semaphore was never created.
		memset(sem, 0, sizeof(os_sem_t));
		ret			= 0;
	}
	else
	{
		// Delete the semaphore.
		knDeleteRtSemaphore(sem->handle);
		memset(sem, 0, sizeof(os_sem_t));
		ret			= 0;
	}

	return(ret);
}



//*****************************************************************************
int os_sem_lock(os_sem_t* sem)
{
	int		ret;
	BOOL	sts;

	if (sem == NULL)
	{
		// The pointer is NULL.
		ret	= -EINVAL;
	}
	else if (sem->key != sem)
	{
		// The semaphore was never created.
		ret	= -EINVAL;
	}
	else
	{
		// Gain access to the semaphore.
		sts	= knWaitForRtSemaphore(sem->handle, 30000);
		ret	= (sts == TRUE) ? 0 : -ERESTARTSYS;
	}

	return(ret);
}



//*****************************************************************************
int os_sem_unlock(os_sem_t* sem)
{
	int	ret;

	if (sem == NULL)
	{
		// The pointer is NULL.
		ret	= -EINVAL;
	}
	else if (sem->key != sem)
	{
		// The semaphore was never created.
		ret	= -EINVAL;
	}
	else
	{
		// Release access to the semaphore.
		knReleaseRtSemaphore(sem->handle);
		ret	= 0;
	}

	return(ret);
}


