// $URL: http://subversion:8080/svn/gsc/trunk/drivers/INtime/sio4/SIO4_INtime_2.x.x.x_DN/drvr_mbox/main.c $
// $Rev: 26115 $
// $Date: 2014-05-18 14:03:04 -0500 (Sun, 18 May 2014) $

#include "main.h"



//*****************************************************************************
static int _parse_args(int argc, char** argv)
{
	char	buf[32];
	int		errs	= 0;
	int		i;

	printf("drvr_mbox - Display the contents of the Driver Mailbox (Version %s)\n", VERSION);
	printf("USAGE: drvr_mbox\n");

	gsc_label_level_inc();

	for (i = 0; i < argc; i++)
	{
		sprintf(buf, "Argument %d", i);
		gsc_label(buf);
		printf("%s\n", argv[i]);
	}

	gsc_label_level_dec();

	for (i = 1; i < argc; i++)
	{
		errs	= 1;
		printf("ERROR: invalid argument: %s\n", argv[i]);
		break;
	}

	return(errs);
}



//*****************************************************************************
static int _dump_driver_mailbox(void)
{
	char		buf[4096];
	int			errs	= 0;
	int			fd;

	gsc_label("Driver Mailbox");
	fd	= sio4_open(-1);

	if (fd == -1)
	{
		errs	= 1;
		printf("FAIL <---  (Unable to open Driver Mailbox.)\n");
	}
	else
	{
		memset(buf, 0, sizeof(buf));
		sio4_read(fd, buf, sizeof(buf));
		buf[sizeof(buf) - 1]	= 0;
		sio4_close(fd);
		printf("\n\n%s\n", buf);
	}

	return(errs);
}



/******************************************************************************
*
*	Function:	main
*
*	Purpose:
*
*		Control the overall operation of the application.
*
*	Arguments:
*
*		argc	The number of command line arguments.
*
*		argv	The list of command line arguments.
*
*	Returned:
*
*		EXIT_SUCCESS	No errors were encounterred.
*		EXIT_FAILURE	One or more errors were encounterred.
*
******************************************************************************/

int main(int argc, char** argv)
{
	int			errs		= 0;
	long		hours;
	long		mins;
	time_t		now;
	int			ret			= EXIT_SUCCESS;
	long		secs;
	time_t		test;

	for (;;)
	{
		gsc_label_init(26);
		test	= time(NULL);
		errs	+= _parse_args(argc, argv);
		os_id_host();

		if (errs == 0)
			errs	+= os_id_driver(SIO4_BASE_NAME);

		if (errs == 0)
			errs	= _dump_driver_mailbox();

		now	= time(NULL);

		if (errs)
		{
			printf(	"\nRESULTS: FAIL <---  (%d error%s)",
					errs,
					(errs == 1) ? "" : "s");
		}
		else
		{
			printf("\nRESULTS: PASS");
		}

		secs	= now - test;
		hours	= secs / 3600;
		secs	= secs % 3600;
		mins	= secs / 60;
		secs	= secs % 60;
		printf(" (duration %ld:%ld:%02ld)\n", hours, mins, secs);
		break;
	}

	if (errs)
		ret	= EXIT_FAILURE;

	return(ret);
}


