// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/id/id.c $
// $Rev: 25701 $
// $Date: 2014-03-31 15:23:50 -0500 (Mon, 31 Mar 2014) $

#include "main.h"



// variables ******************************************************************

static int	_query_list[100];



//*****************************************************************************
static int _query_list_set(int option)
{
	int	errs	= 0;

	if (option >= SIO4_IOCTL_QUERY_LAST)
	{
		errs++;
	}
	else
	{
		errs	= 0;
		_query_list[option]++;
	}

	return(errs);
}



//*****************************************************************************
static int _validate_same_s32(int* fd, s32* val)
{
	int	any		= 0;
	int	errs	= 0;
	s32	good	= 0;
	int	i;

	for (i = 0; i <= 3; i++)
	{
		if (fd[i] == -1)
		{
			continue;
		}
		else if (any == 0)
		{
			any		= 1;
			good	= val[i];
		}
		else if (val[i] != good)
		{
			errs++;
		}
	}

	return(errs);
}



//*****************************************************************************
static int _validate_same_u32(int* fd, u32* val)
{
	int	any		= 0;
	int	errs	= 0;
	u32	good	= 0;
	int	i;

	for (i = 0; i <= 3; i++)
	{
		if (fd[i] == -1)
		{
		}
		else if (any == 0)
		{
			any		= 1;
			good	= val[i];
		}
		else if (val[i] != good)
		{
			errs++;
		}
	}

	return(errs);
}



//*****************************************************************************
static int _show_eol(int errs)
{
	if (errs)
		printf("FAIL <---");

	printf("\n");
	return(errs);
}



//*****************************************************************************
static int _show_s32_long(int* fd, const char* name, s32* val, const char* format)
{
	char	buf[32];
	int		errs	= 0;
	int		i;

	gsc_label(name);

	for (i = 0; i <= 3; i++)
	{
		if (fd[i] == -1)
		{
			local_label_str("----");
			continue;
		}

		sprintf(buf, format, (long) val[i]);
		local_label_str(buf);
	}

	return(errs);
}



//*****************************************************************************
static int _show_u32_hex(int* fd, const char* name, u32* val, const char* format)
{
	char	buf[32];
	int		errs	= 0;
	int		i;

	gsc_label(name);

	for (i = 0; i <= 3; i++)
	{
		if (fd[i] == -1)
		{
			local_label_str("----");
			continue;
		}

		sprintf(buf, format, (long) val[i]);
		local_label_str(buf);
	}

	return(errs);
}



//*****************************************************************************
static int _query_read_s32(int* fd, s32* val, s32 query)
{
	int	errs	= 0;
	int	i;
	int	sts;
	s32	v;

	for (i = 0; i <= 3; i++)
	{
		if (fd[i] != -1)
		{
			v		= query;
			sts		= sio4_ioctl(fd[i], SIO4_IOCTL_QUERY, &v);
			val[i]	= v;
			errs	+= (sts == -1) ? 1 : 0;
		}
	}

	return(errs);
}



//*****************************************************************************
static int _query_read_u32(int* fd, u32* val, s32 query)
{
	int	errs	= 0;
	int	i;
	int	sts;
	u32	v;

	for (i = 0; i <= 3; i++)
	{
		if (fd[i] != -1)
		{
			v		= query;
			sts		= sio4_ioctl(fd[i], SIO4_IOCTL_QUERY, &v);
			val[i]	= v;
			errs	+= (sts == -1) ? 1 : 0;
		}
	}

	return(errs);
}



//*****************************************************************************
static int _query_show_s32(int* fd, s32* val, s32 query, const char* name, const char* format)
{
	int	errs	= 0;
	int	i;
	int	sts;
	s32	v;

	for (i = 0; i <= 3; i++)
	{
		if (fd[i] != -1)
		{
			v		= query;
			sts		= sio4_ioctl(fd[i], SIO4_IOCTL_QUERY, &v);
			val[i]	= v;
			errs	+= (sts == -1) ? 1 : 0;
		}
	}

	errs	+= _show_s32_long(fd, name, val, format);
	return(errs);
}



//*****************************************************************************
static int _show_reg(int* fd, u32* val, u32 reg, const char* name, const char* format)
{
	int	errs	= 0;
	int	i;

	for (i = 0; i <= 3; i++)
	{
		if (fd[i] != -1)
			errs	+= sio4_reg_read(fd[i], reg, &val[i]);
	}

	errs	+= _show_u32_hex(fd, name, val, format);
	return(errs);
}



//*****************************************************************************
static int _show_vidr(int* fd)
{
	int	errs	= 0;
	int	i;
	u32	val[4];

	errs	+= _show_reg(fd, val, GSC_PCI_9080_VIDR, "Vendor ID", "0x%04lX");
	errs	+= _validate_same_u32(fd, val);

	if (errs == 0)
	{
		for (i = 0; i <= 3; i++)
		{
			if (fd[i] == -1)
				continue;

			if (val[i] != 0x10B5)
				errs++;
		}
	}

	_show_eol(errs);
	return(errs);
}



//*****************************************************************************
static int _show_didr(int* fd)
{
	int	errs	= 0;
	int	i;
	u32	val[4];

	errs	+= _show_reg(fd, val, GSC_PCI_9080_DIDR, "Device ID", "0x%04lX");
	errs	+= _validate_same_u32(fd, val);

	if (errs == 0)
	{
		for (i = 0; i <= 3; i++)
		{
			if (fd[i] == -1)
				continue;

			if ((val[i] != 0x9080) && (val[i] != 0x9056))
				errs++;
		}
	}

	_show_eol(errs);
	return(errs);
}



//*****************************************************************************
static int _show_svid(int* fd)
{
	int	errs	= 0;
	int	i;
	u32	val[4];

	errs	+= _show_reg(fd, val, GSC_PCI_9080_SVID, "Sub Vendor ID", "0x%04lX");
	errs	+= _validate_same_u32(fd, val);

	if (errs == 0)
	{
		for (i = 0; i <= 3; i++)
		{
			if (fd[i] == -1)
				continue;

			if (val[i] != 0x10B5)
				errs++;
		}
	}

	_show_eol(errs);
	return(errs);
}



//*****************************************************************************
static int _show_sid(int* fd)
{
	int	errs	= 0;
	int	i;
	u32	val[4];

	errs	+= _show_reg(fd, val, GSC_PCI_9080_SID, "Subsystem ID", "0x%04lX");
	errs	+= _validate_same_u32(fd, val);

	if (errs == 0)
	{
		for (i = 0; i <= 3; i++)
		{
			if (fd[i] == -1)
				continue;

			if ((val[i] != 0x2401) && (val[i] != 0x3198))
				errs++;
		}
	}

	_show_eol(errs);
	return(errs);
}



//*****************************************************************************
static int _show_user_jumper_val(int* fd)
{
	int	errs	= 0;
	s32	val[4];

	errs	+= _query_list_set(SIO4_QUERY_USER_JUMPER_VAL);
	errs	+= _query_show_s32(fd, val, SIO4_QUERY_USER_JUMPER_VAL, "User Jumper Value", "0x%lX");
	errs	+= _validate_same_s32(fd, val);
	_show_eol(errs);
	return(errs);
}



//*****************************************************************************
static int _show_frr(int* fd)
{
	int	errs	= 0;
	u32	val[4];

	errs	+= _show_reg(fd, val, SIO4_GSC_FRR, "Firmware Revision Reg", "0x%08lX");
	errs	+= _validate_same_u32(fd, val);
	_show_eol(errs);
	return(errs);
}



//*****************************************************************************
static int _show_fr(int* fd)
{
	int	errs	= 0;
	u32	val[4];

	errs	+= _show_reg(fd, val, SIO4_GSC_FR, "Features Register", "0x%08lX");
	errs	+= _validate_same_u32(fd, val);
	_show_eol(errs);
	return(errs);
}



//*****************************************************************************
static int _query_bool_str(int* fd, int query, const char* title, int same, const char* yes, const char* no)
{
	char	buf[32];
	int		errs	= 0;
	int		i;
	s32		val[4];

	gsc_label(title);
	errs	+= _query_list_set(query);
	errs	+= _query_read_s32(fd, val, query);

	if (same)
		errs	+= _validate_same_s32(fd, val);

	for (i = 0; i <= 3; i++)
	{
		if (fd[i] == -1)
		{
			local_label_str("----");
			continue;
		}

		switch (val[i])
		{
			default:

				errs++;
				sprintf(buf, "%ld", (long) val[i]);
				local_label_str(buf);
				break;

			case 0:	local_label_str(no);	break;
			case 1:	local_label_str(yes);	break;
		}
	}

	_show_eol(errs);
	return(errs);
}



//*****************************************************************************
static int _query_bool(int* fd, int query, const char* title, int same)
{
	int	errs;

	errs	= _query_bool_str(fd, query, title, same, "Yes", "No");
	return(errs);
}



//*****************************************************************************
static int _query_hex_str(int* fd, int query, const char* title, int same)
{
	char	buf[32];
	int		errs	= 0;
	int		i;
	s32		val[4];

	gsc_label(title);
	errs	+= _query_list_set(query);
	errs	+= _query_read_s32(fd, val, query);

	if (same)
		errs	+= _validate_same_s32(fd, val);

	for (i = 0; i <= 3; i++)
	{
		if (fd[i] == -1)
		{
			local_label_str("----------");
			continue;
		}

		sprintf(buf, "0x%08lX", (long) val[i]);
		local_label_str(buf);
	}

	_show_eol(errs);
	return(errs);
}



//*****************************************************************************
static int _query_hex(int* fd, int query, const char* title, int same)
{
	int	errs;

	errs	= _query_hex_str(fd, query, title, same);
	return(errs);
}



//*****************************************************************************
static int _query_decimal(int* fd, int query, const char* title, const char* suffix, int same)
{
	char	buf[32];
	int		errs	= 0;
	int		i;
	s32		val[4];

	gsc_label(title);
	errs	+= _query_list_set(query);
	errs	+= _query_read_s32(fd, val, query);

	if (same)
		errs	+= _validate_same_s32(fd, val);

	for (i = 0; i <= 3; i++)
	{
		if (fd[i] == -1)
		{
			local_label_str("----");
			continue;
		}

		sprintf(buf, "%ld", (long) val[i]);
		local_label_str(buf);
	}

	if (errs)
		_show_eol(errs);
	else
		printf("%s\n", suffix);

	return(errs);
}



//*****************************************************************************
static int _query_u32(int* fd, int query, const char* title, const char* suffix, const char* format, int same)
{
	char	buf[32];
	int		errs	= 0;
	int		i;
	u32		val[4];

	gsc_label(title);
	errs	+= _query_list_set(query);
	errs	+= _query_read_u32(fd, val, query);

	if (same)
		errs	+= _validate_same_u32(fd, val);

	for (i = 0; i <= 3; i++)
	{
		if (fd[i] == -1)
		{
			local_label_str("----");
			continue;
		}

		sprintf(buf, format, (long) val[i]);
		local_label_str(buf);
	}

	if (errs)
		_show_eol(errs);
	else
		printf("%s\n", suffix);

	return(errs);
}



//*****************************************************************************
static int _query_device_type(int* fd)
{
	char	buf[32];
	int		errs	= 0;
	int		i;
	s32		val[4];

	gsc_label("Device Type");
	errs	+= _query_list_set(SIO4_QUERY_DEVICE_TYPE);
	errs	+= _query_read_s32(fd, val, SIO4_QUERY_DEVICE_TYPE);
	errs	+= _validate_same_s32(fd, val);

	for (i = 0; i <= 3; i++)
	{
		if (fd[i] == -1)
		{
			local_label_str("----");
			continue;
		}

		switch (val[i])
		{
			default:

				errs++;
				sprintf(buf, "%ld", (long) val[i]);
				local_label_str(buf);
				break;

			case GSC_DEV_TYPE_SIO4:			local_label_str("SIO4");		break;
			case GSC_DEV_TYPE_SIO4_SYNC:	local_label_str("SIO4-SYNC");	break;
		}
	}

	_show_eol(errs);
	return(errs);
}



//*****************************************************************************
static int _query_form_factor(int* fd)
{
	int			errs	= 0;
	int			i;
	const char*	psz		= NULL;
	s32			val[4];

	gsc_label("Form Factor");
	errs	+= _query_list_set(SIO4_QUERY_FORM_FACTOR);
	errs	+= _query_read_s32(fd, val, SIO4_QUERY_FORM_FACTOR);
	errs	+= _validate_same_s32(fd, val);

	for (i = 0; i <= 3; i++)
	{
		if (fd[i] == -1)
		{
			local_label_str("----");
			continue;
		}

		errs	+= sio4_id_form_factor(fd[i], &psz, 0);
		local_label_str(psz);
	}

	_show_eol(errs);
	return(errs);
}



//*****************************************************************************
static int _query_sio4_type(int* fd)
{
	int			errs	= 0;
	int			i;
	const char*	psz		= NULL;
	s32			val[4];

	gsc_label("SIO4 Type");
	errs	+= _query_list_set(SIO4_QUERY_SIO4_TYPE);
	errs	+= _query_read_s32(fd, val, SIO4_QUERY_SIO4_TYPE);
	errs	+= _validate_same_s32(fd, val);

	for (i = 0; i <= 3; i++)
	{
		if (fd[i] == -1)
		{
			local_label_str("----");
			continue;
		}

		errs	+= sio4_id_type(fd[i], &psz);
		local_label_str(psz);
	}

	_show_eol(errs);
	return(errs);
}



//*****************************************************************************
static int _query_fw_type(int* fd)
{
	char	buf[32];
	int		errs	= 0;
	int		i;
	s32		val[4];

	gsc_label("Firmware Type");
	errs	+= _query_list_set(SIO4_QUERY_FW_TYPE);
	errs	+= _query_read_s32(fd, val, SIO4_QUERY_FW_TYPE);
	errs	+= _validate_same_s32(fd, val);

	for (i = 0; i <= 3; i++)
	{
		if (fd[i] == -1)
		{
			local_label_str("----");
			continue;
		}

		switch (val[i])
		{
			default:

				errs++;
				sprintf(buf, "%ld", (long) val[i]);
				local_label_str(buf);
				break;

			case SIO4_FW_TYPE_SYNC:		local_label_str("SYNC");	break;
			case SIO4_FW_TYPE_Z16C30:	local_label_str("Z16C30");	break;
		}
	}

	_show_eol(errs);
	return(errs);
}



//*****************************************************************************
static int _query_model_base(int* fd)
{
	int			errs	= 0;
	int			i;
	const char*	psz		= NULL;
	s32			val[4];

	gsc_label("SIO4 Base Model");
	errs	+= _query_list_set(SIO4_QUERY_MODEL_BASE);
	errs	+= _query_read_s32(fd, val, SIO4_QUERY_MODEL_BASE);
	errs	+= _validate_same_s32(fd, val);

	for (i = 0; i <= 3; i++)
	{
		if (fd[i] == -1)
		{
			local_label_str("----");
			continue;
		}

		errs	+= sio4_id_model(fd[i], &psz);
		local_label_str(psz);
	}

	_show_eol(errs);
	return(errs);
}



//*****************************************************************************
static int _query_index_channel(int* fd)
{
	int	errs	= 0;
	int	i;
	s32	val[4];

	errs	+= _query_list_set(SIO4_QUERY_INDEX_CHANNEL);
	errs	+= _query_read_s32(fd, val, SIO4_QUERY_INDEX_CHANNEL);
	errs	+= _show_s32_long(fd, "Channel Index", val, "%ld");

	for (i = 0; i <= 3; i++)
	{
		if (fd[i] == -1)
			continue;

		if (val[i] != i)
			errs++;
	}

	_show_eol(errs);
	return(errs);
}



//*****************************************************************************
static int _query_index_device(int* fd)
{
	int	errs	= 0;
	s32	val[4];

	errs	+= _query_list_set(SIO4_QUERY_INDEX_DEVICE);
	errs	+= _query_read_s32(fd, val, SIO4_QUERY_INDEX_DEVICE);
	errs	+= _show_s32_long(fd, "Device Index", val, "%ld");
	_show_eol(errs);
	return(errs);
}



//*****************************************************************************
static int _query_index_subdevice(int* fd)
{
	int	errs	= 0;
	int	i;
	int	sts;
	s32	v;
	s32	val[4];

	errs	+= _query_list_set(SIO4_QUERY_INDEX_SUBDEVICE);
	errs	+= _query_read_s32(fd, val, SIO4_QUERY_INDEX_SUBDEVICE);
	errs	+= _show_s32_long(fd, "Subdevice Index", val, "%ld");
	errs	+= _validate_same_s32(fd, val);

	for (i = 0; i <= 3; i++)
	{
		if (fd[i] == -1)
			continue;

		if ((val[i] != 0) && (val[i] != 1))
		{
			errs++;
		}
		else
		{
			v	= SIO4_QUERY_DEVICE_QTY;
			sts	= sio4_ioctl(fd[i], SIO4_IOCTL_QUERY, &v);

			if ((sts == -1) || (val[i] > (v - 1)))
				errs++;
		}
	}


	_show_eol(errs);
	return(errs);
}



//*****************************************************************************
static int _query_fifo_size(int* fd, int query, const char* title, int same)
{
	char	buf[32];
	int		errs	= 0;
	s32		fifo;
	int		i;
	s32		qty;
	s32		val[4];

	gsc_label(title);
	errs	+= _query_list_set(query);
	errs	+= _query_read_s32(fd, val, query);
	errs	+= sio4_query(fd[0], SIO4_QUERY_DEVICE_QTY, &qty);

	if (same)
		errs	+= _validate_same_s32(fd, val);

	for (i = 0; i <= 3; i++)
	{
		if (fd[i] == -1)
		{
			local_label_str("----");
			continue;
		}

		fifo	= val[i];

		if ((query == SIO4_QUERY_FIFO_SIZE_TOTAL) && (qty > 1))
			fifo	*= qty;

		if (fifo % 1024)
			sprintf(buf, "%ld", (long) fifo);
		else
			sprintf(buf, "%ldK", (long) fifo / 1024);

		local_label_str(buf);
	}

	if (errs)
		_show_eol(errs);
	else
		printf("bytes\n");

	return(errs);
}



//*****************************************************************************
static int _query_osc_chip(int* fd)
{
	char	buf[32];
	int		errs	= 0;
	int		i;
	s32		val[4];

	gsc_label("OSC Chip Configuration");
	errs	+= _query_list_set(SIO4_QUERY_OSC_CHIP);
	errs	+= _query_read_s32(fd, val, SIO4_QUERY_OSC_CHIP);
	errs	+= _validate_same_s32(fd, val);

	for (i = 0; i <= 3; i++)
	{
		if (fd[i] == -1)
		{
			local_label_str("----");
			continue;
		}

		switch (val[i])
		{
			default:

				errs++;
				sprintf(buf, "%ld", (long) val[i]);
				local_label_str(buf);
				break;

			case SIO4_OSC_CHIP_FIXED:		local_label_str("Fixed");			break;
			case SIO4_OSC_CHIP_IDC2053B:	local_label_str("IDC2053B (1)");	break;
			case SIO4_OSC_CHIP_IDC2053B_4:	local_label_str("IDC2053B (4)");	break;
			case SIO4_OSC_CHIP_CY22393:		local_label_str("CY22393 (1)");		break;
			case SIO4_OSC_CHIP_CY22393_2:	local_label_str("CY22393 (2)");		break;
		}
	}

	_show_eol(errs);
	return(errs);
}



//*****************************************************************************
static int _query_mp_chip(int* fd)
{
	char	buf[32];
	int		errs	= 0;
	int		i;
	s32		val[4];

	gsc_label("MP Chip Configuration");
	errs	+= _query_list_set(SIO4_QUERY_MP_CHIP);
	errs	+= _query_read_s32(fd, val, SIO4_QUERY_MP_CHIP);
	errs	+= _validate_same_s32(fd, val);

	for (i = 0; i <= 3; i++)
	{
		if (fd[i] == -1)
		{
			local_label_str("----");
			continue;
		}

		switch (val[i])
		{
			default:

				errs++;
				sprintf(buf, "%ld", (long) val[i]);
				local_label_str(buf);
				break;

			case SIO4_MP_CHIP_FIXED:	local_label_str("Fixed");	break;
			case SIO4_MP_CHIP_SP508:	local_label_str("SP508");	break;
		}
	}

	_show_eol(errs);
	return(errs);
}



//*****************************************************************************
static int _query_count(int* fd)
{
	char	buf[32];
	char	err[64]	= "";
	int		errs	= 0;
	int		i;
	int		j;
	int		sts;
	s32		v;
	s32		val[4];

	gsc_label("Query Count");
	errs	+= _query_list_set(SIO4_QUERY_COUNT);

	if ((errs) && (err[0] == 0))
		sprintf(err, "%d. _query_list_set", __LINE__);

	errs	+= _query_read_s32(fd, val, SIO4_QUERY_COUNT);

	if ((errs) && (err[0] == 0))
		sprintf(err, "%d. _query_read_s32", __LINE__);

	errs	+= _validate_same_s32(fd, val);

	if ((errs) && (err[0] == 0))
		sprintf(err, "%d. _validate_same_s32", __LINE__);

	for (i = 0; i <= 3; i++)
	{
		if (fd[i] == -1)
		{
			local_label_str("----");
			continue;
		}

		sprintf(buf, "%ld", (long) val[i]);
		local_label_str(buf);

		if (val[i] != SIO4_IOCTL_QUERY_LAST)
		{
			errs++;

			if (err[0] == 0)
			{
				sprintf(err,
						"%d. incorrect count, ch[%d], expected %ld",
						__LINE__,
						i,
						(long) SIO4_IOCTL_QUERY_LAST);
			}
		}

		// Verify that each query passes.

		for (j = 0; j < val[i]; j++)
		{
			v	= j;
			sts	= sio4_ioctl(fd[i], SIO4_IOCTL_QUERY, &v);

			if (sts == -1)
			{
				errs++;

				if (err[0] == 0)
				{
					sprintf(err,
							"%d. SIO4_IOCTL_QUERY request[%d]",
							__LINE__,
							j);
				}
			}

			if ((j == SIO4_QUERY_REG_PSRCR_BITS) && (v == -1))
				continue;

			if ((j == SIO4_QUERY_REG_PSTSR_BITS) && (v == -1))
				continue;

			if (v == -1)
			{
				errs++;

				if (err[0] == 0)
				{
					sprintf(err,
							"%d. SIO4_IOCTL_QUERY request[%d] results",
							__LINE__,
							j);
				}
			}
		}
	}

	if (errs)
		printf("FAIL <---");

	if ((errs) && (err[0]))
		printf("  (%s)", err);

	printf("\n");
	return(errs);
}



//*****************************************************************************
static int _query_list_validate(void)
{
	int	err;
	int	errs	= 0;
	int	i;

	// Missed entries.

	for (err = 0, i = 0; i < SIO4_IOCTL_QUERY_LAST; i++)
	{
		if (_query_list[i] == 0)
		{
			if (err == 0)
			{
				err++;
				gsc_label("Query Options Missed");
				printf("FAIL <---  (%d", i);
			}
			else
			{
				err++;
				printf(", %d", i);
			}
		}
	}

	if (err)
	{
		errs++;
		printf(")\n");
	}

	// Duplicate entries.

	for (err = 0, i = 0; i <= SIO4_IOCTL_QUERY_LAST; i++)
	{
		if (_query_list[i] > 1)
		{
			if (err == 0)
			{
				err++;
				gsc_label("Query Options Duplicated");
				printf("FAIL <---  (%d", i);
			}
			else
			{
				err++;
				printf(", %d", i);
			}
		}
	}

	if (err)
	{
		errs++;
		printf(")\n");
	}

	return(errs);
}



//*****************************************************************************
static int _show_zilog_rev(int* fd)
{
	char	buf[32];
	int		err;
	int		errs	= 0;
	int		i;
	u32		val[4]	= { 0x4444, 0x4444, 0x4444, 0x4444 };
	s32		z16c30[4];

	gsc_label("Zilog Z16C30 Rev");
	errs	+= _query_read_s32(fd, z16c30, SIO4_QUERY_MODEL_Z16C30);
	errs	+= _validate_same_s32(fd, z16c30);

	for (i = 0; i <= 3; i++)
	{
		if ((fd[i] == -1) || (z16c30[i] == 0))
		{
			local_label_str("----");
			val[i]	= 0x4444;
			continue;
		}

		err		= 0;
		err		+= sio4_reg_write(fd[i], SIO4_USC_TMCR, 0x1F);
		err		+= sio4_reg_read(fd[i], SIO4_USC_TMDR, &val[i]);
		errs	+= err;

		if (err)
		{
			local_label_str("FAIL <---");
			val[i]	= 0x4444;
			continue;
		}

		sprintf(buf, "0x%04lX", (long) val[i]);
		local_label_str(buf);

		if (val[i] != 0x4444)
			errs++;
	}

	errs	+= _validate_same_u32(fd, val);
	_show_eol(errs);
	return(errs);
}



//*****************************************************************************
static int _register_map_list_gsc(int* fd)
{
	char					buf[32];
	int						ch;
	int						err;
	int						errs	= 0;
	u32						format;
	int						i;
	const gsc_reg_def_t*	def;
	int						fd_tmp[4];
	u32						fw;
	u32						mult;
	s32						supp[4];
	u32						val[4]	= { 0, 0, 0, 0 };

	printf("\n");

	for (i = 0;; i++)
	{
		err	= 0;
		def	= sio4_reg_get_def_index(i);

		if (def == NULL)
			break;

		fw		= SIO4_FIRMWARE_DECODE(def->reg);
		mult	= SIO4_CH_MULT_DECODE(def->reg);
		format	= SIO4_FORMAT_DECODE(def->reg);

		if (fw == 0)
			break;

		gsc_label(def->name);

		// Read the register values.

		for (ch = 0; ch <= 3; ch++)
		{
			if (fd[ch] != -1)
				err	+= sio4_reg_read(fd[ch], def->reg, &val[ch]);
		}

		// Report the register values.
		fd_tmp[0]	= fd[0];
		fd_tmp[1]	= fd[1];
		fd_tmp[2]	= fd[2];
		fd_tmp[3]	= fd[3];

		for (ch = 0; ch <= 3; ch++)
		{
			if (fd[ch] == -1)
			{
				local_label_str("----");
				continue;
			}

			supp[ch]	= 1;

			if (def->ask_support)
			{
				supp[ch]	= (def->decode)(fd[ch], 1, 0, 0);

				if (supp[ch] == 0)
				{
					fd_tmp[ch]	= -1;	// don't compare value if unsupported
					local_label_str("NA");
				}
			}

			if (supp[ch])
			{
				sprintf(buf, "0x%08lX", (long) val[ch]);
				local_label_str(buf);
			}

			if ((mult) || (format))
				fd_tmp[ch]	= -1;	// don't compare value if this type reg

			if (def->reg == SIO4_GSC_GPIOSR)
				fd_tmp[ch]	= -1;	// don't compare values of this register

			if (def->reg == SIO4_GSC_TSR)
				fd_tmp[ch]	= -1;	// don't compare values of this register
		}

		// Perform channel value comparisons.

		if (def->ask_support)
			err	+= _validate_same_s32(fd, supp);

		err	+= _validate_same_u32(fd_tmp, val);

		if (err)
			printf("FAIL <---\n");
		else
			printf("%s\n", def->desc);

		errs	+= err;
	}

	return(errs);
}



//*****************************************************************************
static int _register_map_list_usc(int* fd)
{
	char					buf[32];
	int						ch;
	int						err;
	int						errs	= 0;
	int						i;
	const gsc_reg_def_t*	def;
	u32						fw;
	u32						val[4]	= { 0, 0, 0, 0 };

	printf("\n");

	for (i = 0;; i++)
	{
		err	= 0;
		def	= sio4_reg_get_def_index(i);

		if (def == NULL)
			break;

		fw	= SIO4_FIRMWARE_DECODE(def->reg);

		if (fw)
			continue;

		gsc_label(def->name);

		// Read the register values.

		for (ch = 0; ch <= 3; ch++)
		{
			if (fd[ch] != -1)
				err	+= sio4_reg_read(fd[ch], def->reg, &val[ch]);
		}

		// Report the register values.

		for (ch = 0; ch <= 3; ch++)
		{
			if (fd[ch] == -1)
			{
				local_label_str("----");
				continue;
			}

			sprintf(buf, "0x%04lX", (long) val[ch]);
			local_label_str(buf);
		}

		// Perform channel value comparisons.

		if (err)
			printf("FAIL <---\n");
		else
			printf("%s\n", def->desc);

		errs	+= err;
	}

	return(errs);
}



//*****************************************************************************
static int _frr_detail(int fd)
{
	gsc_reg_def_t			list[2]	= { { NULL }, { NULL } };
	const gsc_reg_def_t*	ptr;

	int	errs	= 0;

	printf("\n");
	ptr	= sio4_reg_get_def_id(SIO4_GSC_FRR);

	if (ptr)
	{
		list[0]	= ptr[0];
		errs	= gsc_reg_list(fd, list, 1, sio4_reg_read);
	}
	else
	{
		errs++;
		printf(	"FAIL <---  (%d. sio4_reg_get_def_id, SIO4_GSC_FRR)\n",
				__LINE__);
	}

	return(errs);
}



//*****************************************************************************
static int _fr_detail(int fd)
{
	gsc_reg_def_t			list[2]	= { { NULL }, { NULL } };
	const gsc_reg_def_t*	ptr;

	int	errs	= 0;

	printf("\n");
	ptr	= sio4_reg_get_def_id(SIO4_GSC_FR);

	if (ptr)
	{
		list[0]	= ptr[0];
		errs	= gsc_reg_list(fd, list, 1, sio4_reg_read);
	}
	else
	{
		errs++;
		printf(	"FAIL <---  (%d. sio4_reg_get_def_id, SIO4_GSC_FR)\n",
				__LINE__);
	}

	return(errs);
}



//*****************************************************************************
int id_device(int* fd)
{
	int	errs	= 0;
	int	i;
	s32	z16c30;

	// Identification =====================================
	gsc_label("Identification");
	printf("\n");
	gsc_label_level_inc();

	errs	+= _show_vidr(fd);
	errs	+= _show_didr(fd);
	errs	+= _show_svid(fd);
	errs	+= _show_sid(fd);
	errs	+= _show_user_jumper_val(fd);
	errs	+= _show_frr(fd);
	errs	+= _show_fr(fd);

	gsc_label_level_dec();

	// Features ==============================================
	gsc_label("Device Features");
	printf("\n");
	memset(_query_list, 0, sizeof(_query_list));

	gsc_label_level_inc();
	errs	+= _query_device_type(fd);
	errs	+= _query_form_factor(fd);
	errs	+= _query_sio4_type	(fd);
	errs	+= _query_fw_type	(fd);
	errs	+= _query_bool		(fd, SIO4_QUERY_MODEL_SYNC,				"SYNC Model Board",					1	);
	errs	+= _query_bool		(fd, SIO4_QUERY_MODEL_Z16C30,			"Z16C30 Model Board",				1	);
	errs	+= _query_model_base(fd);
	errs	+= _query_decimal	(fd, SIO4_QUERY_INDEX_BOARD,			"Board Index",			"",			1	);
	errs	+= _query_index_device(fd);
	errs	+= _query_index_subdevice(fd);
	errs	+= _query_index_channel(fd);
	errs	+= _query_fifo_size	(fd, SIO4_QUERY_FIFO_SIZE_TOTAL,		"Total FIFO Size",					1	);
	errs	+= _query_fifo_size	(fd, SIO4_QUERY_FIFO_SIZE_RX,			"Rx FIFO Size",						0	);
	errs	+= _query_fifo_size	(fd, SIO4_QUERY_FIFO_SIZE_TX,			"Tx FIFO Size",						0	);
	errs	+= _query_bool		(fd, SIO4_QUERY_RX_FIFO_FULL_CFG,		"Rx FIFO Full Config",				1	);
	errs	+= _query_bool		(fd, SIO4_QUERY_RX_FIFO_FULL_CFG_GLB,	"Rx FIFO Full Config Glb",			1	);
	errs	+= _query_bool		(fd, SIO4_QUERY_RX_FIFO_OVERRUN,		"Rx FIFO Overrun",					1	);
	errs	+= _query_bool		(fd, SIO4_QUERY_RX_FIFO_UNDERRUN,		"Rx FIFO Underrun",					1	);
	errs	+= _query_bool		(fd, SIO4_QUERY_TX_FIFO_EMPTY_CFG,		"Tx FIFO Empty Config",				1	);
	errs	+= _query_bool		(fd, SIO4_QUERY_TX_FIFO_OVERRUN,		"Tx FIFO Overrun",					1	);
	errs	+= _query_bool		(fd, SIO4_QUERY_FIFO_SPACE_CFG,			"FIFO Space Config",				1	);
	errs	+= _query_decimal	(fd, SIO4_QUERY_USER_JUMPER_QTY,		"User Jumpers",			"",			1	);
	errs	+= _query_bool_str	(fd, SIO4_QUERY_USER_JUMPER_SENSE,		"User Jumper Sense", 1, "1 = On", "0 = On");
	errs	+= _show_user_jumper_val(fd);

	errs	+= _query_bool		(fd, SIO4_QUERY_FW_TYPE_CONFIG,			"FW Type Config",					1	);

	errs	+= _query_decimal	(fd, SIO4_QUERY_BUS_SPEED,				"PCI Bus Speed",		"MHz",		1	);
	errs	+= _query_decimal	(fd, SIO4_QUERY_BUS_WIDTH,				"PCI Bus Width",		"bits",		1	);
	errs	+= _query_decimal	(fd, SIO4_QUERY_DEVICE_QTY,				"Board Devices",		"",			1	);
	errs	+= _query_decimal	(fd, SIO4_QUERY_CHANNEL_QTY,			"Board Channels",		"",			1	);

	errs	+= _query_decimal	(fd, SIO4_QUERY_LED_CHANNEL,			"Channel LEDs",			"",			1	);
	errs	+= _query_decimal	(fd, SIO4_QUERY_LED_MAIN,				"Main LEDs",			"",			1	);

	errs	+= _query_osc_chip	(fd);
	errs	+= _query_bool		(fd, SIO4_QUERY_OSC_PER_CHANNEL,		"Osc Per Channel",					1	);
	errs	+= _query_bool		(fd, SIO4_QUERY_OSC_PROGRAM,			"Osc Programmable",					1	);
	errs	+= _query_decimal	(fd, SIO4_QUERY_OSC_PD_MAX,				"Osc Max Post Divider",	"",			1	);
	errs	+= _query_bool		(fd, SIO4_QUERY_OSC_MEASURE,			"Osc Measure",						1	);

	errs	+= _query_bool		(fd, SIO4_QUERY_MP,						"MP (Multi-Protocol)",				1	);
	errs	+= _query_mp_chip	(fd);
	errs	+= _query_bool		(fd, SIO4_QUERY_MP_PROGRAM,				"MP Programmable",					1	);
	errs	+= _query_hex		(fd, SIO4_QUERY_MP_BITMAP,				"MP Protocol Bitmap",				1	);

	errs	+= _query_bool		(fd, SIO4_QUERY_BOARD_RESET,			"Board Reset",						1	);
	errs	+= _query_bool		(fd, SIO4_QUERY_RX_STATUS_WORD,			"Rx Status Word Enable",			1	);
	errs	+= _query_bool		(fd, SIO4_QUERY_TIME_STAMP,				"Timestamp",						1	);

	errs	+= _query_bool_str	(fd, SIO4_QUERY_IRQ_32,					"32 IRQ Bits", 1, "Yes (32)", "No (16)");
	errs	+= _query_bool		(fd, SIO4_QUERY_LEGACY_CABLE,			"Legacy Cable API",					1	);
	errs	+= _query_bool		(fd, SIO4_QUERY_DMDMA_SCD,				"DMDMA Sin Cyc Dis",				1	);
	errs	+= _query_count		(fd);

	errs	+= _query_bool		(fd, SIO4_QUERY_REG_BSR,				"Reg: BSR",							1	);
	errs	+= _query_bool		(fd, SIO4_QUERY_REG_CCR,				"Reg: CCR",							1	);
	errs	+= _query_bool		(fd, SIO4_QUERY_REG_FCR,				"Reg: FCR",							1	);
	errs	+= _query_bool		(fd, SIO4_QUERY_REG_FR,					"Reg: FR",							1	);
	errs	+= _query_bool		(fd, SIO4_QUERY_REG_FSR,				"Reg: FSR",							1	);
	errs	+= _query_bool		(fd, SIO4_QUERY_REG_FTR,				"Reg: FTR",							1	);
	errs	+= _query_bool		(fd, SIO4_QUERY_REG_GPIOSR,				"Reg: GPIOSR",						1	);
	errs	+= _query_bool		(fd, SIO4_QUERY_REG_IELR,				"Reg: IELR",						1	);
	errs	+= _query_bool		(fd, SIO4_QUERY_REG_IHLR,				"Reg: IHLR",						1	);
	errs	+= _query_bool		(fd, SIO4_QUERY_REG_IOCR,				"Reg: IOCR",						1	);
	errs	+= _query_bool		(fd, SIO4_QUERY_REG_PCR,				"Reg: PCR",							1	);
	errs	+= _query_bool		(fd, SIO4_QUERY_REG_POCSR,				"Reg: POCSR",						1	);
	errs	+= _query_bool		(fd, SIO4_QUERY_REG_PORAR,				"Reg: PORAR",						1	);
	errs	+= _query_bool		(fd, SIO4_QUERY_REG_PORDR,				"Reg: PORDR",						1	);
	errs	+= _query_bool		(fd, SIO4_QUERY_REG_PORD2R,				"Reg: PORD2R",						1	);
	errs	+= _query_bool		(fd, SIO4_QUERY_REG_PSRCR,				"Reg: PSRCR",						1	);
	errs	+= _query_u32		(fd, SIO4_QUERY_REG_PSRCR_BITS,			"Reg: PSRCR Bits",	"", "0x%08lX",	1	);
	errs	+= _query_bool		(fd, SIO4_QUERY_REG_PSTSR,				"Reg: PSTSR",						1	);
	errs	+= _query_u32		(fd, SIO4_QUERY_REG_PSTSR_BITS,			"Reg: PSTSR Bits",	"", "0x%08lX",	1	);
	errs	+= _query_bool		(fd, SIO4_QUERY_REG_RCR,				"Reg: RCR",							1	);
	errs	+= _query_bool		(fd, SIO4_QUERY_REG_SBR,				"Reg: SBR",							1	);
	errs	+= _query_bool		(fd, SIO4_QUERY_REG_TCR,				"Reg: TCR",							1	);
	errs	+= _query_bool		(fd, SIO4_QUERY_REG_TSR,				"Reg: TSR",							1	);

	gsc_label_level_dec();

	errs	+= _query_list_validate();

	errs	+= _show_zilog_rev(fd);

	z16c30	= 0;

	for (i = 0; i <= 3; i++)
	{
		if (fd[i] != -1)
		{
			errs	+= sio4_query(fd[i], SIO4_QUERY_MODEL_Z16C30, &z16c30);
			break;
		}
	}

	errs	+= _register_map_list_gsc(fd);

	if (z16c30)
		errs	+= _register_map_list_usc(fd);

	for (i = 0; i <= 3; i++)
	{
		if (fd[i] != -1)
		{
			errs	+= _frr_detail(fd[i]);
			errs	+= _fr_detail(fd[i]);
			break;
		}
	}

	printf("\n");
	return(errs);
}


