// $URL: http://subversion:8080/svn/gsc/trunk/drivers/gsc_common/lib/intime/os_ioctl.c $
// $Rev: 26099 $
// $Date: 2014-05-16 10:51:54 -0500 (Fri, 16 May 2014) $

// INtime 4 library

#include "main.h"



//*****************************************************************************
// returns err_no: 0 = ok, !=0 = error value
static int _ioctl_large(os_file_t* fp, int cmd, void* buf, int bytes)
{
	int			err_no	= 0;
	os_mbox_t*	mbox	= NULL;
	RTHANDLE	mem		= BAD_RTHANDLE;
	os_msg_t*	msg		= NULL;
	u32			offset	= 0;
	int			sem		= BAD_RTHANDLE;
	WORD		size;
	DWORD		sts;

	for (;;)
	{
		// Acquire access to the structure.
		sem	= os_sem_lock(&fp->sem);

		if (sem)
		{
			errno	= sem;
			break;
		}

		// Acquire a mailbox for the receive message.
		err_no	= os_mbox_alloc(&mbox);

		if (err_no)
			break;

		// Allocate a message structure.
		err_no	= os_msg_alloc(&msg);

		if (err_no)
			break;

		// Create a memory handle for the buffer.
		err_no	= os_mem_handle_create(buf, bytes, &offset, &mem);

		if (err_no)
			break;

		// Initialize and send the message.
		memset(msg, 0, sizeof(os_msg_t));
		msg->status				= OS_MSG_STS_OK;
		msg->msg_type			= OS_MSG_TYPE_IOCTL_LG;
		msg->size				= OS_MSG_SIZE_IOCTL_LG;
		msg->mbox				= mbox->mbox;
		msg->u.ioctl_lg.cmd		= cmd;
		msg->u.ioctl_lg.mem		= (u32) mem;
		msg->u.ioctl_lg.offset	= offset;
		sts						= SendRtData(fp->dev_mbox, msg, OS_MSG_SIZE_IOCTL_LG);

		if (sts == FALSE)
		{
			err_no	= GetLastRtError();
			break;
		}

		// Get and process the response.
		os_sem_unlock(&fp->sem);
		sem		= -1;
		size	= ReceiveRtData(mbox->mbox, msg, WAIT_FOREVER);

		if ((size != OS_MSG_SIZE_IOCTL_LG) || (size != msg->size))
		{
			err_no	= EIO;
			break;
		}

		switch (msg->status)
		{
			default:

				err_no	= EBADMSG;
				break;

			case OS_MSG_STS_OK:

				err_no	= msg->u.ioctl_lg.err_no;
				break;

			case OS_MSG_STS_MSG_SIZE:

				err_no	= EMSGSIZE;
				break;

			case OS_MSG_STS_MSG_TYPE:

				err_no	= EBADMSG;
				break;

			case OS_MSG_STS_MEM_MAP:

				err_no	= ENOMEM;
				break;

			case OS_MSG_STS_THREAD:

				err_no	= ENOEXEC;
				break;
		}

		break;
	}

	os_mem_handle_destroy(&mem);

	if (msg)
		os_msg_free(&msg);

	os_mbox_free(&mbox);

	if (sem == 0)
		os_sem_unlock(&fp->sem);

	return(err_no);
}



//*****************************************************************************
// returns err_no: 0 = ok, !=0 = error value
static int _ioctl_small(os_file_t* fp, int cmd, void* buf, int buf_size)
{
	int			err_no		= 0;
	os_mbox_t*	mbox		= NULL;
	os_msg_t*	msg			= NULL;
	int			msg_size	= OS_MSG_SIZE_IOCTL_SM_MIN + buf_size;
	int			sem			= BAD_RTHANDLE;
	WORD		size;
	DWORD		sts;

	for (;;)
	{
		// Acquire access to the structure.
		sem	= os_sem_lock(&fp->sem);

		if (sem)
		{
			errno	= sem;
			break;
		}

		// Acquire a mailbox for the receive message.
		err_no	= os_mbox_alloc(&mbox);

		if (err_no)
			break;

		// Allocate a message structure.
		err_no	= os_msg_alloc(&msg);

		if (err_no)
			break;

		// Initialize and send the message.
		memset(msg, 0, sizeof(os_msg_t));

		if (cmd & OS_IOCTL_DIR_WRITE)
			memcpy(msg->u.ioctl_sm.buf, buf, buf_size);

		msg->status			= OS_MSG_STS_OK;
		msg->msg_type		= OS_MSG_TYPE_IOCTL_SM;
		msg->size			= msg_size;
		msg->mbox			= mbox->mbox;
		msg->u.ioctl_sm.cmd	= cmd;
		sts					= SendRtData(fp->dev_mbox, msg, msg_size);

		if (sts == FALSE)
		{
			err_no	= GetLastRtError();
			break;
		}

		// Get and process the response.
		os_sem_unlock(&fp->sem);
		sem		= -1;
		size	= ReceiveRtData(mbox->mbox, msg, WAIT_FOREVER);

		if (size != msg_size)
		{
			err_no	= EIO;
			break;
		}

		if (cmd & OS_IOCTL_DIR_READ)
			memcpy(buf, msg->u.ioctl_sm.buf, buf_size);

		switch (msg->status)
		{
			default:

				err_no	= EBADMSG;
				break;

			case OS_MSG_STS_OK:

				err_no	= msg->u.ioctl_sm.err_no;
				break;

			case OS_MSG_STS_MSG_SIZE:

				err_no	= EMSGSIZE;
				break;

			case OS_MSG_STS_MSG_TYPE:

				err_no	= EBADMSG;
				break;

			case OS_MSG_STS_MEM_MAP:

				err_no	= ENOMEM;
				break;

			case OS_MSG_STS_THREAD:

				err_no	= ENOEXEC;
				break;
		}

		break;
	}

	if (msg)
		os_msg_free(&msg);

	os_mbox_free(&mbox);

	if (sem == 0)
		os_sem_unlock(&fp->sem);

	return(err_no);
}



//*****************************************************************************
// returns err_no: 0 = ok, !=0 = error value
int os_ioctl(int fd, int cmd, void* buf)
{
	int			err_no;
	os_file_t*	fp		= (os_file_t*) fd;
	os_msg_t*	msg		= NULL;
	int			size	= OS_IOCTL_SIZE_DECODE(cmd);

	if ((fd <= 0) || (fp->val_ptr != fp) || (buf == NULL) || (size <= 0))
	{
		err_no	= EINVAL;
	}
	else if (size <= sizeof(msg->u.ioctl_sm.buf))
	{
		err_no	= _ioctl_small(fp, cmd, buf, size);
	}
	else
	{
		err_no	= _ioctl_large(fp, cmd, buf, size);
	}

	return(err_no);
}


