// $URL: http://subversion:8080/svn/gsc/trunk/drivers/gsc_common/utils/intime/os_util_id.c $
// $Rev: 23633 $
// $Date: 2013-10-22 11:15:26 -0500 (Tue, 22 Oct 2013) $

#include "sio4_lib.h"
#include "gsc_utils.h"



// #defines *******************************************************************

#define	ISEOL(_c)	((((_c) == 0)		|| \
					  ((_c) == '\r')	|| \
					  ((_c) == '\n')) ? 1 : 0)



//*****************************************************************************
static void _id_host_os(void)
{
	SYSINFO	si;

	gsc_label("Operating System");
	printf("INtine for Windows\n");

	gsc_label("Kernel");
	CopyRtSystemInfo(&si);
	printf(	"%ld.%ld, Build %ld\n",
			(long) si.MajorVersion,
			(long) si.MinorVersion,
			(long) si.BuildNumber);
}



//*****************************************************************************
static void _id_host_processor(void)
{
	const char*	psz;
	SYSINFO		si;

	CopyRtSystemInfo(&si);

	gsc_label("Processor");
	printf(	"CPU Type %ld, CPU Model %ld\n",
			(long) si.CPUType,
			(long) si.CPUModel);

	switch (si.NodeType)
	{
		default:						psz	= "INTERNAL ERROR";	break;
		case NODE_TYPE_SINGLE_CPU:		psz	= "Single CPU";		break;
		case NODE_TYPE_SHARED_MP:		psz	= "Shared MP";		break;
		case NODE_TYPE_DEDICATED_MP:	psz	= "Dedicated MP";	break;
		case NODE_TYPE_REMOTE_MP:		psz	= "Remote MP";		break;
	}

	gsc_label("Node Type");
	printf("%s\n", psz);
}



/******************************************************************************
*
*	Function:	os_id_host
*
*	Purpose:
*
*		Identify the host os and machine.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		None.
*
******************************************************************************/

void os_id_host(void)
{
	gsc_label("Host");
	printf("\n");
	gsc_label_level_inc();

	_id_host_os();
	_id_host_processor();

	gsc_label_level_dec();
}





/******************************************************************************
*
*	Function:	os_id_driver
*
*	Purpose:
*
*		Identify the driver.
*
*	Arguments:
*
*		base	The base name of the device.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int os_id_driver(const char* base)
{
	char*	blt;
	char	buf[1024];
	int		errs	= 0;
	int		fd;
	char*	ver;

	for (;;)	// A convenience loop.
	{
		gsc_label("Driver");
		fd	= sio4_open(-1);

		if (fd == -1)
		{
			errs++;
			printf("FAIL <---  (Driver mailbox access error.)\n");
			break;
		}

		memset(buf, 0, sizeof(buf));
		sio4_read(fd, buf, sizeof(buf));
		buf[sizeof(buf) - 1]	= 0;
		sio4_close(fd);

		ver	= strstr(buf, "version");
		blt	= strstr(buf, "built");

		if ((ver == NULL) || (blt == NULL))
		{
			errs++;
			printf("FAIL <---  (Driver mailbox content error.)\n");
			break;
		}

		// Version
		printf("Version ");

		for (ver += 9; !ISEOL(ver[0]); ver++)
			printf("%c", ver[0]);

		printf(", Built ");

		for (blt += 7; !ISEOL(blt[0]); blt++)
			printf("%c", blt[0]);

		printf("\n");
		break;
	}

	return(errs);
}


